@@ -1,9 +1,9 @@
 package Dist::Zilla::Plugin::OurPkgVersion;
-use 5.008;
+use 5.014;
 use strict;
 use warnings;
 
-our $VERSION = '0.11'; # VERSION
+our $VERSION = '0.15'; # VERSION
 
 use Moose;
 with (
@@ -11,11 +11,7 @@ with (
 	'Dist::Zilla::Role::FileFinderUser' => {
         default_finders => [
             ':InstallModules',
-            (
-                eval { Dist::Zilla::Dist::Builder->VERSION('5.038'); 1 }
-                    ? ':PerlExecFiles'
-                    : ':ExecFiles'
-            )
+            ':PerlExecFiles',
         ],
 	},
 	'Dist::Zilla::Role::PPI',
@@ -26,10 +22,31 @@ use PPI;
 use MooseX::Types::Perl qw( LaxVersionStr );
 use namespace::autoclean;
 
+has underscore_eval_version => (
+  is      => 'ro',
+  isa     => 'Int',
+  default => 0,
+);
+
+has skip_main_module => (
+  is      => 'ro',
+  isa     => 'Int',
+  default => 0,
+);
+
+has overwrite => (
+  is      => 'ro',
+  isa     => 'Int',
+  default => 0,
+);
+
 sub munge_files {
 	my $self = shift;
 
-	$self->munge_file($_) for @{ $self->found_files };
+	$self->munge_file($_) for
+		grep { $self->skip_main_module ? $_->name ne $self->zilla->main_module->name : 1 }
+		@{ $self->found_files };
+
 	return;
 }
 
@@ -50,12 +67,13 @@ sub munge_file {
 
 	return unless defined $doc;
 
+	$doc->index_locations if $self->overwrite; # optimize calls to check line numbers
 	my $comments = $doc->find('PPI::Token::Comment');
 
 	my $version_regex
 		= q{
                   ^
-                  (\s*)              # capture all whitespace before comment
+                  (\s*)              # capture leading whitespace for whole-line comments
                   (
                     \#\#?\s*VERSION  # capture # VERSION or ## VERSION
                     \b               # and ensure it ends on a word boundary
@@ -71,7 +89,7 @@ sub munge_file {
 	if ( ref($comments) eq 'ARRAY' ) {
 		foreach ( @{ $comments } ) {
 			if ( /$version_regex/xms ) {
-				my ( $ws, $comment ) =  ( $1, $2 );
+				my ( $ws, $comment ) = ( $1, $2 );
 				$comment =~ s/(?=\bVERSION\b)/TRIAL /x if $self->zilla->is_trial;
 				my $code
 						= "$ws"
@@ -79,7 +97,24 @@ sub munge_file {
 						. $version
 						. qq{'; $comment}
 						;
-				$_->set_content("$code");
+				# If the comment is not a whole-line comment, and if the user wants to overwrite
+				# existing "our $VERSION=...;", then find the other tokens from this line, looking
+				# for our $VERSION = $VALUE.  If found, edit only the VALUE.
+				if ( $self->overwrite && !$_->line ) {
+					my $line_no = $_->line_number;
+					my $nodes = $doc->find( sub { $_[1]->line_number == $line_no } );
+					my $version_value_token = $nodes && $self->_identify_version_value_token(@$nodes);
+					if ( $version_value_token ) {
+						$version_value_token->set_content(qq{'$version'});
+						$code = $ws . $comment;
+						$munged_version++;
+					}
+				}
+				if ( $version =~ /_/ && $self->underscore_eval_version ) {
+					my $eval = "\$VERSION = eval \$VERSION;";
+					$code .= $_->line? "$eval\n" : "\n$eval";
+				}
+				$_->set_content($code);
 				$munged_version++;
 			}
 		}
@@ -97,6 +132,33 @@ sub munge_file {
 	}
 	return;
 }
+
+sub _identify_version_value_token {
+	my ( $self, @tokens ) = @_;
+	my $val_tok;
+	my @want = ('our', '$VERSION', '=', undef, ';');
+	my $i = 0;
+	for ( @tokens ) {
+		next if $_->isa('PPI::Token::Whitespace');
+		# If the next thing we want is "undef", this is where we capture the value token.
+		if (!defined $want[$i]) {
+			$val_tok = $_;
+			++$i;
+		}
+		# Else if the token matches the current step in the sequence, advance the sequence
+		# If sequence completely matched, return.
+		elsif ($want[$i] eq $_->content) {
+			++$i;
+			return $val_tok if $i >= $#want;
+		}
+		# A mismatch restarts the search
+		elsif ($i) {
+			$i = 0;
+		}
+	}
+	return; # no match
+}
+
 __PACKAGE__->meta->make_immutable;
 1;
 # ABSTRACT: No line insertion and does Package version with our
@@ -111,7 +173,7 @@ Dist::Zilla::Plugin::OurPkgVersion - No line insertion and does Package version
 
 =head1 VERSION
 
-version 0.11
+version 0.15
 
 =head1 SYNOPSIS
 
@@ -222,6 +284,8 @@ becomes
 
 	our $VERSION = '0.01'; # TRIAL VERSION
 
+=encoding UTF-8
+
 =head1 METHODS
 
 =over
@@ -245,6 +309,37 @@ files are properly marked as executables for the installer.
 
 =back
 
+=head1 PROPERTIES
+
+=over
+
+=item underscore_eval_version
+
+For version numbers that have an underscore in them, add this line
+immediately after the our version assignment:
+
+ $VERSION = eval $VERSION;
+
+This is arguably the correct thing to do, but changes the line numbering
+of the generated Perl module or source, and thus optional.
+
+=item skip_main_module
+
+Set to true to ignore the main module in the distribution. This prevents
+a warning when using L<Dist::Zilla::Plugin::VersionFromMainModule> to
+obtain the version number instead of the C<dist.ini> file.
+
+=item overwrite
+
+When enabled, this option causes any match of the C<< # VERSION >> comment
+to first check for an existing C<< our $VERSION = ...; >> on the same line,
+and if found, overwrite the value in the existing statement. (the comment
+still gets modified for trial releases)
+
+Currently, the value must be a single Perl token such as a string or number.
+
+=back
+
 =head1 BUGS
 
 Please report any bugs or feature requests on the bugtracker website
@@ -256,7 +351,7 @@ feature.
 
 =head1 CONTRIBUTORS
 
-=for stopwords Alexandr Ciornii Alexei Znamensky Christian Walde Christopher J. Madsen David Golden Graham Ollis Ian Sealy Stephan Loyd
+=for stopwords Alexandr Ciornii Michael Jemmeson Stephan Loyd Alexei Znamensky Christian Walde Christopher J. Madsen David Golden Graham Ollis Graham✈️✈️ Ian Sealy Conrad
 
 =over 4
 
@@ -266,6 +361,14 @@ Alexandr Ciornii <alexchorny@gmail.com>
 
 =item *
 
+Michael Jemmeson <mjemmeson@cpan.org>
+
+=item *
+
+Stephan Loyd <stephanloyd9@gmail.com>
+
+=item *
+
 Alexei Znamensky <russoz@cpan.org>
 
 =item *
@@ -290,11 +393,15 @@ Graham Ollis <plicease@cpan.org>
 
 =item *
 
+Graham✈️✈️ <plicease@cpan.org>
+
+=item *
+
 Ian Sealy <git@iansealy.com>
 
 =item *
 
-Stephan Loyd <stephanloyd9@gmail.com>
+Michael Conrad <mike@nrdvana.net>
 
 =back
 
@@ -314,7 +421,7 @@ Grahan Ollis <plicease@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2017 by Caleb Cushing.
+This software is Copyright (c) 2019 by Caleb Cushing.
 
 This is free software, licensed under:
 
